#coding:utf-8
'''
Модуль с различными полезными окнами и прочими элементами
пользовательского интерфейса.

Created on 24.02.2011

@author: akvarats
'''

from m3_ext.ui import (windows,
                       fields,
                       controls,
                       containers,)

class ExternalLinkWindow(windows.ExtWindow):
    '''
    Окно, в котором есть внешняя ссылка + некоторое описание
    того, что эта ссылка означает.
    '''
    def __init__(self, title='', link_html='', link_style={}, desc_html='', desc_style={}, *args, **kwargs):
        super(ExternalLinkWindow, self).__init__(*args, **kwargs)

        self.title = title or u'Внешняя ссылка'
        self.width = 400
        self.height = 300

        self.layout = 'form'

        self.container_link = containers.ExtContainer(html=link_html,
                                                      style=link_style)

        self.container_description = containers.ExtContainer(html=desc_html,
                                                             style=desc_style)

        self.items.extend([self.container_link,
                            self.container_description,])

        self.buttons.append(controls.ExtButton(text=u'Закрыть', handler='closeWindow'))



#coding:utf-8
"""
Created on 21.04.2010

@author: prefer <telepenin@bars-open.ru>
"""

from m3_ext.ui.base import ExtUIComponent
from containers import ExtContainer
from base import BaseExtContainer


class ExtContainerTable(BaseExtContainer):
    """
    Контейнерный компонент.
    Имеет в себе табличную настройку (строки и колонки)
    и позволяет в ячейках указывать произвольные контролы,
    которые будут помещены в ExtContainer с layput=form
    """
    _DEFAULT_HEIGHT = 36

    def __init__(self, columns=0, rows=0, *args, **kwargs):
        """
        :param columns: Количество колонок
        :type columns: int
        :param rows: Количество строк
        :type rows: int
        """
        super(ExtContainerTable, self).__init__(*args, **kwargs)
        self.template = 'ext-containers/ext-container.js'

        # Заголовок
        self.title = None

        self.__columns_count = 0
        self.__rows_count = 0
        self.__table = []
        self.__rows_height = {}

        # Количество колонок
        self.columns_count = columns

        # Количество строк
        self.rows_count = rows

        self.init_component(*args, **kwargs)

    def _init_properties(self):
        """
        Первоначальное заполнение матрицы пустыми словарями.
        .. note::
            Вложенный словарь с произвольными свойствами для каждой ячейки,
            например {1: {2: {'width': 100}}},
            где 1-номер колонки, 2-номер строки.
        """
        self._properties = {}
        for col_num in range(self.__columns_count):
            d = dict([(row_num, {}) for row_num in range(self.__rows_count)])
            self._properties[col_num] = d

    def render(self):
        for row_num, row in enumerate(self.__table):
            col_cont_list = []
            for col_num, col in enumerate(row):
                if col is not None:
                    if isinstance(col, int):
                        col = ExtContainer(layout='form', flex=1)
                    elif not isinstance(col, ExtContainer):
                        raise Exception('Unknown type of column "%s"' % col)

                    col_cont_list.append(col)

                    # Устанавливаем произвольные свойства для колонки,
                    # если они есть
                    props = self._properties[col_num][row_num]
                    for key, value in props.items():
                        setattr(col, key, value)

            height = self.__rows_height.get(row_num) or (
                ExtContainerTable._DEFAULT_HEIGHT)
            row_cont = ExtContainer(
                layout_config=dict(align="stretch"),
                layout='hbox',
                height=height
            )
            row_cont.items.extend(col_cont_list)
            self._items.append(row_cont)

        return super(ExtContainerTable, self).render()

    def set_properties(self, row_num=None, col_num=None, **kwargs):
        """
        Устанавливает свойство контейнера в заданной колонке и(или) строке.
        :param col_num: Номер колонки. Если не задано, то вся колонка.
        :type col_num: int
        :param row_num: Номер строки. Если не задано, то вся строка.
        :type row_num: int

        :raise: AssertionError
        """
        assert col_num is None or 0 <= col_num <= self.columns_count, (
            'Number %s more than the number of columns %s' % (
                col_num, self.columns_count
            )
        )
        assert row_num is None or 0 <= row_num <= self.rows_count, (
            'Number %s more than the number of rows %s' % (
                row_num, self.rows_count
            )
        )
        if col_num is not None and row_num is not None:
            self._properties[col_num][row_num].update(kwargs)
        # Задана только колонка
        elif col_num is not None:
            for d in self._properties[col_num].values():
                d.update(kwargs)
        # Задана только строка
        elif row_num is not None:
            for d in self._properties.values():
                d[row_num].update(kwargs)

    def _make_read_only(
            self, access_off=True, exclude_list=(), *args, **kwargs):
        for item in self.items:
            item.make_read_only(
                access_off, exclude_list, *args, **kwargs)

    @property
    def items(self):
        return [
            col
            for row in self.__table
            for col in row
            if isinstance(col, ExtContainer)
        ]

    @property
    def columns_count(self):
        return self.__columns_count

    @columns_count.setter
    def columns_count(self, value):
        assert isinstance(value, int), 'Value must be INT'
        self.__columns_count = value
        self._init_properties()
        if self.__rows_count:
            self.__init_table()

    @property
    def rows_count(self):
        return self.__rows_count

    @rows_count.setter
    def rows_count(self, value):
        assert isinstance(value, int), 'Value must be INT'
        self.__rows_count = value
        self._init_properties()
        if self.__columns_count:
            self.__init_table()

    def __init_table(self):
        self.__table = [
            range(self.__columns_count)
            for col in range(self.__rows_count)
        ]

    def find_by_name(self, name):
        for item in self.items:
            if hasattr(item, 'name') and name == item.name:
                return item

            if hasattr(item, 'find_by_name'):
                res = item.find_by_name(name)
                if res:
                    return res

    def set_item(self, row, col, cmp, colspan=1, **kwargs):
        """
        Устанавливает контрол *cmp* в ячейку с колонкой *col* и строкой *row*
        :param row: строка
        :type row: int
        :param col: колонка
        :type col: int
        :param cmp: контрол
        :type cmp: BaseExtUIComponent или наследник
        :param colspan: сколько строк будет занимать компонент
        :type colspan: int
        :param **kwargs: свойства контейнера
        """
        assert isinstance(cmp, ExtUIComponent)
        assert isinstance(colspan, int)
        cont = ExtContainer(
            layout='form',
            flex=colspan,
            style=dict(padding="0px")
        )
        # добавляем отступ слева, если это уже не первая колонка
        if col != 0:
            cont.style = dict(padding="0px 0px 0px 5px")
        cmp.anchor = '100%'
        cont.items.append(cmp)
        self.__table[row][col] = cont
        self.set_properties(row, col, **kwargs)
        if colspan > 1:
            self.__table[row][col + 1: col + colspan] = [None] * (colspan - 1)

    def set_row_height(self, row, height):
        """
        Устанавливает ширину и высоту ячейки
        :param row: Индекс ячейки
        :type row: int
        :param height: Высота
        :type height: int
        """
        assert isinstance(height, int), 'Height must be INT'
        assert isinstance(row, int), 'Row num must be INT'
        assert 0 <= row <= self.rows_count, (
            'Row num %d must be in range 0 to %d' % (row, self.rows_count))
        self.__rows_height[row] = height

    def set_default_row_height(self, row):
        """
        Выставляет умолчательные
        (_DEFAULT_HEIGHT) параметры ширины и высоты ячейки
        :param row: Индекс Ячейки
        :type row: int
        """
        assert isinstance(row, int), 'Row num must be INT'
        assert 0 <= row <= self.rows_count, (
            'Row num must be in range 0 to %d' % self.rows_count)
        self.__rows_height[row] = ExtContainerTable._DEFAULT_HEIGHT

    def set_rows_height(self, height):
        """
        Устанавливает у всех строк высоту
        :param height: Высота
        :type height: int
        """
        assert isinstance(height, int), 'Height must be INT'
        for row in range(self.rows_count):
            self.__rows_height[row] = height

    def set_default_rows_height(self):
        """
        Устанавливает умолчательные параметры у всех ячеек
        """
        self.set_rows_height(ExtContainerTable._DEFAULT_HEIGHT)



#coding:utf-8
"""
Created on 27.02.2010

@author: akvarats
"""

from m3_ext.ui.base import ExtUIComponent

from m3_ext.ui.misc import ExtDataStore


class BaseExtField(ExtUIComponent):
    """
    Базовый класс для полей
    """
    def __init__(self, *args, **kwargs):
        super(BaseExtField, self).__init__(*args, **kwargs)
        # Нужно выставлять пустое значение для того, чтобы обязательные поля,
        # те, которые allow_blank=False подсвечивались автоматически после
        # рендеринга
        self.value = ""

        # Признак нередактируемости поля
        self.read_only = False

        # Признак, что поле используется для изменения значения,
        # а не для навигации
        # - при Истине будут повешаны обработчики на изменение окна
        # см. m3.js
        self.is_edit = True

        # Не обязательно для заполнения (True),
        # иначе поле должно быть не пустым
        self.allow_blank = True

        # Тип валидации
        self.vtype = None

        # Этот текст будет выводиться, если поле незаполненно
        self.empty_text = None

        # Минимальные длина поля и текст ошибки, если длина была превышена
        self.min_length = self.min_length_text = None

        # Максимальные длина поля и текст ошибки, если длина была превышена
        self.max_length = self.max_length_text = None

        # Валидация на регулярное вырожение и текст ошибки,
        # если валидация будет нарушена
        self.regex = self.regex_text = None

        # Порядок обхода для этого поля
        self.tab_index = None

        # Свой CSS класс валидации для некорректно заполненного поля
        # TODO: Вынести в атрибут класса, а не атрибут экземпляра
        self.invalid_class = 'm3-form-invalid'

        # Текст, который будет отображаться, если поле заполненно некорректно
        self.invalid_text = None

        # Плагины к полям ввода
        self.plugins = []

        # Дополнительные DOM-атрибуты
        self.auto_create = {
            "tag": "input",
            "type": "text",
            "size": "20",
            "autocomplete": "off"
        }

    def t_render_label_style(self):
        if isinstance(self.label_style, dict):
            return ';'.join(
                ['%s:%s' % (k, v) for k, v in self.label_style.items()])
        else:
            return self.label_style

    def t_render_regex(self):
        return '/%s/' % self.regex

    def _make_read_only(
            self, access_off=True, exclude_list=(), *args, **kwargs):
        if self in exclude_list or self.name in exclude_list:
            return
        self.read_only = access_off
        # Выключаем/включаем обязательность заполнения.
        if not hasattr(self, '_allow_blank_old'):
            self._allow_blank_old = self.allow_blank
        self.allow_blank = True if access_off else self._allow_blank_old

    #==========================================================================
    # Врапперы над событиями listeners[...]
    #==========================================================================
    @property
    def handler_specialkey(self):
        return self._listeners.get('specialkey')

    @handler_specialkey.setter
    def handler_specialkey(self, function):
        self._listeners['specialkey'] = function

    @property
    def handler_change(self):
        return self._listeners.get('change')

    @handler_change.setter
    def handler_change(self, function):
        self._listeners['change'] = function

    def render_base_config(self):
        if self.read_only:
            grey_cls = 'm3-grey-field'
            self.cls = grey_cls if not self.cls else self.cls + grey_cls

        super(BaseExtField, self).render_base_config()

        for args in (
            ('value', self.value),
            ('readOnly', self.read_only, self.read_only),
            ('isEdit', self.is_edit),
            ('allowBlank', self.allow_blank, not self.allow_blank),
            ('vtype', self.vtype),
            ('emptyText', self.empty_text),
            ('minLength', self.min_length),
            ('minLengthText', self.min_length_text),
            ('maxLength', self.max_length),
            ('maxLengthText', self.max_length_text),
            ('regex', self.t_render_regex, self.regex),
            ('regexText', self.regex_text),
            ('tabIndex', self.tab_index),
            ('invalidClass', self.invalid_class),
            ('invalidText', self.invalid_text),
            ('plugins',
                (lambda: '[%s]' % ','.join(self.plugins)), self.plugins),
        ):
            self._put_config_value(*args)

        # дополнительно вешаем DOM-атрибуты через Ext.Field.autoCreate
        if self.max_length:
            self.auto_create.update({"maxlength": self.max_length})
            self._put_config_value('autoCreate', self.auto_create)


class BaseExtTriggerField(BaseExtField):
    """
    Базовый класс для комбобокса, поля выбора справочника
    """

    ALL = 'all'
    QUERY = 'query'

    def __init__(self, *args, **kwargs):
        super(BaseExtTriggerField, self).__init__(*args, **kwargs)

        # Поле, которое будет отображаться при выборе
        self.display_field = None

        # Поле, которое будет использоваться в качестве значения
        self.value_field = None

        #
        self.hidden_name = None

        # Скрыть триггера выподающего списка
        self.hide_trigger = False

        #
        self.type_ahead = False

        #
        self.query_param = None

        # Количество записей, показываемых на странице
        self.page_size = None

        # Максимальная высота выподаюего списка
        self.max_heigth_dropdown_list = None

        # Количество введенных символов,
        # после которых произойдет запрос на сервер
        self.min_chars = None

        # Ссылка на хранилище данных
        self.__store = None

        #
        self.mode = None

        # Признак возможности редактирования
        self.editable = True

        # Признак, что отображаться при выборе будут все записи (QUERY),
        # Иначе те, которые подходят введенному тексту (ALL)
        self.trigger_action = BaseExtTriggerField.QUERY

        #
        self.force_selection = False

        # Текст, если записей в сторе нет
        self.not_found_text = None

        # Текст, отображаемый при загрузке данных
        self.loading_text = u'Загрузка...'

        # иные имена полей (кроме id и display_field),
        # которые будут попадать в store
        self.fields = []

        # ширина выпадающего списка
        self.list_width = None

        # шаблон рендера выпадающего списка
        self.list_tpl = None

        # изменение ширины выпадающего списка
        self.resizable = False

    def set_store(self, store):
        self.mode = 'local' if isinstance(store, ExtDataStore) else 'remote'
        self.__store = store

    def get_store(self):
        return self.__store

    store = property(get_store, set_store)

    def t_render_store(self):
        assert self.__store, 'Store is not define'
        # отрисуем стор с полями
        if self.display_field in self.fields:
            return self.__store.render(self.fields)
        else:
            return self.__store.render([self.display_field]+self.fields)

    @property
    def trigger_action_all(self):
        """
        Для обратной совместимости
        """
        return self.trigger_action == BaseExtTriggerField.ALL

    @trigger_action_all.setter
    def trigger_action_all(self, value):
        """
        Для обратной совместимости
        """
        self.trigger_action = BaseExtTriggerField.ALL if value else \
            BaseExtTriggerField.QUERY

    @property
    def name(self):
        return self.hidden_name

    @name.setter
    def name(self, value):
        self.hidden_name = value

    @property
    def handler_change(self):
        return self._listeners.get('change')

    @handler_change.setter
    def handler_change(self, function):
        self._listeners['change'] = function

    @property
    def handler_select(self):
        return self._listeners.get('select')

    @handler_select.setter
    def handler_select(self, function):
        self._listeners['select'] = function

    @property
    def handler_afterrender(self):
        return self._listeners.get('afterrender')

    @handler_afterrender.setter
    def handler_afterrender(self, function):
        self._listeners['afterrender'] = function

    def pre_render(self):
        if self.get_store():
            self.get_store().action_context = self.action_context
        super(BaseExtTriggerField, self).pre_render()

    def render_base_config(self):
        self.pre_render()

        super(BaseExtTriggerField, self).render_base_config()

        for args in (
            ('displayField', self.display_field),
            ('valueField', self.value_field),
            ('hiddenName', self.hidden_name),
            ('hideTrigger', self.hide_trigger),
            ('typeAhead', self.type_ahead),
            ('queryParam', self.query_param),
            ('pageSize', self.page_size),
            ('maxHeight', self.max_heigth_dropdown_list),
            ('minChars', self.min_chars),
            ('mode', self.mode),
            ('triggerAction', self.trigger_action),
            ('editable', self.editable),
            ('forceSelection', self.force_selection),
            ('valueNotFoundText', self.not_found_text),
            ('loadingText', self.loading_text),
            ('store', self.t_render_store, self.get_store()),
            ('listWidth', self.list_width, self.list_width),
            ('tpl', self.list_tpl, self.list_tpl),
            ('resizable', self.resizable, self.resizable),
        ):
            self._put_config_value(*args)



# coding: utf-8
from logging import getLogger
import datetime
import decimal
import os
import json

try:
    from PIL import Image  # require PIL module
except ImportError:
    class _FakeImage(object):
        """
        Заглушка для PIL.Image
        """
        def __getattr__(self, attr):
            raise ImportError("PIL not installed!")
    Image = _FakeImage

from django.core.files.base import ContentFile
from django.conf import settings

from m3_django_compat import ModelOptions
from m3.actions.interfaces import ISelectablePack, IMultiSelectablePack
from m3_ext.ui.base import BaseExtComponent
from m3_ext.ui.containers.base import BaseExtPanel

from ..helpers import _render_globals


logger = getLogger('django')


def _is_field(obj):
    from m3_ext.ui.fields.base import BaseExtField
    return isinstance(obj, BaseExtField)


class ExtForm(BaseExtPanel):
    """
    Форма, умеющая биндиться и делать сабмит по урлу
    """

    def __init__(self, *args, **kwargs):
        super(ExtForm, self).__init__(*args, **kwargs)
        self.template = 'ext-panels/ext-form.js'

        # Свой layout
        self.layout = 'form'

        # Отступ от внешнего контрола
        self.padding = None

        # url для сабмита
        self.url = None

        # Будут ли загружаться файлы
        self.file_upload = False

        # поле, которое будет под фокусом ввода после рендеринга формы
        self.focused_field = None

        self.init_component(*args, **kwargs)

    def _get_all_fields(self, item, lst=None):
        """
        Возвращает список всех полей формы включая вложенные в контейнеры
        """
        if lst is None:
            lst = []
        if _is_field(item):
            lst.append(item)

        elif hasattr(item, 'items'):
            for it in item.items:
                self._get_all_fields(it, lst)
        return lst

    def bind_to_request(self, request):
        """
        Извлекает из запроса параметры и присваивает их соответствующим полям
        формы
        """
        assert request, 'Request must be define!'

        from m3_ext.ui import fields

        all_fields = self._get_all_fields(self)
        for field in all_fields:
            name = field.name
            if isinstance(field, (
                fields.ExtFileUploadField,
                fields.ExtImageUploadField
            )):
                # Файлы нужно забирать из request.FILES
                field.memory_file = request.FILES.get(
                    fields.ExtFileUploadField.PREFIX + field.name)
            # возьмем только то, что есть в запросе
            if name in request.POST:
                value = request.POST.get(name)
                field.value = value

    #TODO необходимо добавить проверку на возникновение exception'ов
    def from_object(self, obj, exclusion=None):
        """
        Метод выполнения прямого связывания данных атрибутов объекта obj и
        полей текущей формы
        """
        from m3_ext.ui import fields
        from m3_ext.ui.fields import simple

        exclusion = exclusion or []

        def is_secret_token(value):
            """
            Возвращает истину если значение поля содержит секретный ключ с
            персональной информацией. Он не должен биндится,
            т.к. предназначен для обработки в personal.middleware
            """
            return unicode(value)[:2] == u'##'

        def _assign_value(value, item):
            """
            Конвертирует и присваивает значение value в соответствии типу item.
            """
            if isinstance(item, fields.ExtStringField):
                if value:
                    item.value = unicode(value)
                else:
                    item.value = u''

            elif isinstance(item, simple.ExtAdvTimeField):
                item.value = '%02d:%02d:%02d' % (
                    value.hour, value.minute, value.second
                ) if not is_secret_token(value) else unicode(value)

            elif isinstance(item, simple.ExtDateTimeField):
                if isinstance(value, datetime.datetime):
                    item.value = '%02d.%02d.%04d %02d:%02d:%02d' % (
                        value.day, value.month, value.year, value.hour,
                        value.minute, value.second
                    )
                elif isinstance(value, datetime.date):
                    item.value = '%02d.%02d.%04d 00:00:00' % (
                        value.day, value.month, value.year
                    )
                else:
                    item.value = value

            elif isinstance(item, fields.ExtDateField):
                if isinstance(value, (datetime.date, datetime.datetime)):
                    item.value = '%02d.%02d.%04d' % (
                        value.day, value.month, value.year
                    ) if not is_secret_token(value) else unicode(value)
                else:
                    item.value = value

            elif isinstance(item, fields.ExtTimeField):
                if isinstance(value, (datetime.time, datetime.datetime)):
                    item.value = '%02d:%02d' % (
                        value.hour, value.minute
                    ) if not is_secret_token(value) else unicode(value)
                else:
                    item.value = value

            elif isinstance(item, fields.ExtCheckBox):
                item.checked = True if value else False
            elif isinstance(item, fields.ExtRadio):
                item.checked = (value == item.value)
            elif isinstance(item, fields.ExtMultiSelectField):
                # У поля выбора может быть сзязанный с ним пак
                bind_pack = getattr(item, 'pack', None) or getattr(
                    item, 'bind_pack', None)
                if bind_pack:
                    assert isinstance(bind_pack, IMultiSelectablePack), (
                        'Pack %s must provide IMultiSelectablePack interface' %
                        bind_pack
                    )
                    item.value = bind_pack.get_display_dict(
                        value,
                        value_field=item.value_field,
                        display_field=item.display_field
                    )
            elif isinstance(item, fields.ExtDictSelectField):
                # У поля выбора может быть сзязанный с ним пак
                # TODO после окончательного удаления метода
                # configure_by_dictpack в ExtDictSelectField
                # нужно удалить проверку на 'bind_pack'
                bind_pack = getattr(item, 'pack', None) or getattr(
                    item, 'bind_pack', None)
                if bind_pack and value:
                    assert isinstance(bind_pack, ISelectablePack), (
                        'Pack %s must provide ISelectablePack interface' %
                        bind_pack
                    )
                    if hasattr(bind_pack, 'get_record'):
                        record = bind_pack.get_record(value)
                        if record:
                            item.set_value_from_model(record)
                        # TODO: возможно, здесь нужно возбуждать исключение
                        # else:
                        #     raise bind_pack.model.DoesNotExist()
                    else:
                        item.default_text = bind_pack.get_display_text(
                            value, item.display_field)
                item.value = value
            elif isinstance(item, fields.ExtComboBox) and hasattr(
                    item, 'bind_rule_reverse'):
                # Комбобокс как правило передает id выбранного значения.
                # Его не так просто  преобразовать в тип объекта,
                # Поэтому нужно использовать либо трансляцию значений,
                # либо вызывать специальную функцию
                # внутри экземпляра комбобокса.
                if callable(item.bind_rule_reverse):
                    item.value = unicode(item.bind_rule_reverse(value))
                elif isinstance(item.bind_rule_reverse, dict):
                    item.value = unicode(item.bind_rule_reverse.get(value))
                else:
                    raise ValueError(
                        'Invalid attribute type bind_rule_reverse.'
                        'Must be a function or a dict.'
                    )

            elif isinstance(item, (
                fields.ExtFileUploadField,
                fields.ExtImageUploadField
            )):
                item.value = unicode(value)
                # Относительную URL ссылку до статики
                if value:
                    item.file_url = value.url.lstrip('/')
                else:
                    item.file_url = None

                # Прибиндим оригинальные размеры thumbnail
                if isinstance(item, fields.ExtImageUploadField):
                    if hasattr(settings, 'MEDIA_ROOT') and item.thumbnail:
                        ffile = os.path.join(
                            settings.MEDIA_ROOT, unicode(value))
                        dir_ = os.path.dirname(ffile)
                        file_name = os.path.basename(ffile)

                        thumb_file = os.path.join(
                            dir_,
                            (fields.ExtImageUploadField.MIN_THUMBNAIL_PREFIX +
                                file_name))
                        if os.path.exists(thumb_file):
                            thumb = Image.open(thumb_file)
                            item.thumbnail_size = thumb.size

            else:
                item.value = unicode(value)

        def get_value(obj, names):
            """
            Ищет в объекте obj поле с именем names и возвращает его значение.
            Если соответствующего поля не оказалось, то возвращает None

            names задается в виде списка, т.о. если его длина больше единицы,
            то имеются вложенные объекты и их надо обработать
            :param obj: объект или словарь
            :type obj: object или dict
            :param names: список имен
            :type names: list
            """

            # hasattr не работает для dict'a
            has_attr = (
                hasattr(obj, names[0])
                if not isinstance(obj, dict) else names[0] in obj)
            if has_attr:
                if len(names) == 1:
                    if isinstance(obj, dict):
                        return obj[names[0]], True
                    else:
                        value = getattr(obj, names[0])
                        return value() if callable(value) else value, True
                else:
                    nested = (
                        getattr(obj, names[0])
                        if not isinstance(obj, dict) else obj[names[0]])
                    return get_value(nested, names[1:])
            return None, False

        all_fields = self._get_all_fields(self)
        for field in all_fields:
            if not field.name:
                continue
            field_name = str(field.name)
            assert field_name, (
                'The names of all fields must be set for a successful '
                'assignment. Check the definition of the form.'
            )
            # заполним атрибуты только те, которые не в списке исключаемых
            if field_name not in exclusion:
                names = field_name.split('.')
                new_val, has_attr = get_value(obj, names)
                if has_attr:
                    new_val = new_val if new_val is not None else ''
                    _assign_value(new_val, field)

    #TODO необходимо добавить проверку на возникновение exception'ов
    def to_object(self, obj, exclusion=None):
        '''
        Метод выполнения обратного связывания данных.
        :param obj: объект
        :type obj: object
        :param exclusion: список исключаемых аттрибутов
        :type exclusion: list
        '''
        from m3_ext.ui import fields
        from m3_ext.ui.fields import simple

        exclusion = exclusion or []

        def save_image(obj, name, field):
            """
            Работа с изображением или файлом
            :param field: поле
            :type field: потомок m3_ext.ui.fields.complex.BaseExtField
            :param obj: объект
            :param name: имя поля
            :type name: str
            """
            if hasattr(obj, name):
                l_field = getattr(obj, name)
                if (
                    l_field and
                    os.path.exists(l_field.path) and
                    (
                        os.path.basename(l_field.file.name)
                    ).lower() != (field.value).lower()
                ):
                    # Сначало нужно удалить thumbnail картинки
                    if isinstance(field, fields.ExtImageUploadField) and (
                            field.thumbnail):
                        current_dir = os.path.dirname(l_field.path)
                        basename = os.path.basename(l_field.path)

                        thumb_prefix = (
                            fields.ExtImageUploadField.MIDDLE_THUMBNAIL_PREFIX,
                            fields.ExtImageUploadField.MIN_THUMBNAIL_PREFIX,
                            fields.ExtImageUploadField.MAX_THUMBNAIL_PREFIX
                        )

                        for prefix in thumb_prefix:
                            thumb = os.path.join(
                                current_dir, prefix + basename)

                            if os.path.exists(thumb):
                                os.remove(thumb)

                    # Файл изменился, удаляем старый
                    l_field.delete(save=False)

                if field.memory_file:
                    cont_file = ContentFile(field.memory_file.read())
                    name_file = field.memory_file.name

                    l_field = getattr(obj, name)
                    l_field.save(name_file, cont_file, save=False)

                    if isinstance(field, fields.ExtImageUploadField):
                        try:
                            img = Image.open(l_field.path)
                        except IOError:
                            # Кроме логирования ничего не нужно
                            logger.exception()
                            return

                        width, height = img.size
                        max_width, max_height = field.image_max_size

                        # Обрезаем изображение, если нужно
                        if width > max_width or height > max_height:

                            # curr_width, curr_height = get_img_size(
                            #     field.image_max_size, img.size)
                            # new_img = img.resize(
                            #     (curr_width, curr_height), Image.ANTIALIAS)
                            img.save(l_field.path)

                        if isinstance(field, fields.ExtImageUploadField) and (
                                field.thumbnail and field.memory_file):
                            current_dir = os.path.dirname(l_field.path)

                            # А так же нужно сохранять thumbnail картинки
                            # Состовляем лист thumbnail_size'ов
                            thumbnails = [(
                                field.min_thumbnail_size,
                                fields.ExtImageUploadField.MIN_THUMBNAIL_PREFIX
                            )]

                            if field.middle_thumbnail_size:
                                thumbnails.append((
                                    field.middle_thumbnail_size,
                                    fields.ExtImageUploadField.MIDDLE_THUMBNAIL_PREFIX
                                ))

                            if field.max_thumbnail_size:
                                thumbnails.append((
                                    field.max_thumbnail_size,
                                    fields.ExtImageUploadField.MAX_THUMBNAIL_PREFIX
                                ))

                            for thumb in thumbnails:
                                # Генерируем thumbnails
                                # thumb_zise = get_img_size(size, img.size)
                                # new_img = img.resize(
                                #     thumb_zise, Image.ANTIALIAS)

                                base_name = os.path.basename(l_field.path)
                                tmb_path = os.path.join(
                                    current_dir, thumb[1] + base_name)
                                img.save(tmb_path)

        def set_field(obj, names, value, field=None):
            """
            Ищет в объекте obj поле с именем names
            и присваивает значение value.
            Если соответствующего поля не оказалось, то оно не создается

            names задается в виде списка, т.о. если его длина больше единицы,
            то имеются вложенные объекты
            :param obj: объект
            :param names: список имен
            :param value: значение
            :param field: поле
            :type field: потомок m3_ext.ui.fields.complex.BaseExtField
            """

            # hasattr не работает для dict'a
            has_attr = (
                hasattr(obj, names[0])
                if not isinstance(obj, dict) else names[0] in obj)
            if has_attr:
                if len(names) == 1:
                    if isinstance(obj, dict):
                        obj[names[0]] = value
                    elif isinstance(field, (
                        fields.ExtFileUploadField,
                        fields.ExtImageUploadField
                    )):
                        save_image(obj, names[0], field)

                    else:
                        # Для id нельзя присваивать пустое значение!
                        # Иначе модели не будет сохраняться
                        if names[0] == 'id' and value == '':
                            return

                        setattr(obj, names[0], value)
                else:
                    nested = (
                        getattr(obj, names[0])
                        if not isinstance(obj, dict) else obj[names[0]])
                    set_field(nested, names[1:], value, field)

        def try_to_int(value, default=None):
            """
            Пробует преобразовать value в целое число,
            иначе возвращает default
            :param value: значение
            :param default: значение, возвращаемое, если value не указано
            :rtype: int
            """
            if not value:
                return default

            try:
                return int(value)
            except ValueError:
                return value

        def try_to_list(value):
            """
            Пробует преобразовать value в список
            :param value: значение
            :rtype: list
            """
            if not value:
                return []

            try:
                return json.loads(value)
            except ValueError:
                return value

        def convert_value(item):
            """
            Берет значение item.value,
            и конвертирует его в соответствии с типом item'a
            :type item: потомок m3_ext.ui.fields.complex.BaseExtField
            """
            val = item.value
            if isinstance(item, fields.ExtNumberField):
                if val:
                    # для языков, где decimal разделяются не точкой
                    if item.decimal_separator:
                        val = val.replace(item.decimal_separator, '.')
                    try:
                        val = int(val)
                    except ValueError:
                        try:
                            val = decimal.Decimal(val)
                        except decimal.InvalidOperation:
                            val = None
                else:
                    val = None
            elif isinstance(item, fields.ExtStringField):
                val = unicode(val) if val is not None else None
            elif isinstance(item, simple.ExtAdvTimeField):
                if val and val.strip():
                    d = datetime.datetime.strptime(val, "%H:%M:%S")
                    val = d.time()
                else:
                    val = None
            elif isinstance(item, simple.ExtDateTimeField):
                if val and val.strip():
                    val = datetime.datetime.strptime(val, '%d.%m.%Y %H:%M:%S')
                else:
                    val = None
            elif isinstance(item, fields.ExtDateField):
                #TODO уточнить формат дат
                if val and val.strip():
                    d = datetime.datetime.strptime(val, '%d.%m.%Y')
                    val = d.date()
                else:
                    val = None
            elif isinstance(item, fields.ExtTimeField):
                if val and val.strip():
                    d = datetime.datetime.strptime(val, '%H:%M')
                    val = datetime.time(d.hour, d.minute, 0)
                else:
                    val = None
            elif isinstance(item, fields.ExtCheckBox):
                val = True if val == 'on' else False
            elif isinstance(item, fields.ExtComboBox):
                # Комбобокс как правило передает id выбранного значения.
                #Его не так просто преобразовать в тип объекта,
                # т.к. мы ничего не знаем о структуре объекта.
                # Поэтому нужно использовать либо трансляцию значений,
                # либо вызывать специальную функцию внутри
                # экземпляра комбобокса.
                if hasattr(item, 'bind_rule'):
                    if callable(item.bind_rule):
                        val = item.bind_rule(val)
                    elif isinstance(item.bind_rule, dict):
                        val = item.bind_rule.get(val)
                    else:
                        raise ValueError(
                            'Invalid attribute type bind_rule.'
                            'Must be a function or a dict.'
                        )
                else:
                    val = try_to_int(val)

            elif isinstance(item, fields.ExtMultiSelectField):
                val = try_to_list(val)

            elif isinstance(item, fields.ExtDictSelectField):
                val = try_to_int(val)

            elif isinstance(item, fields.ExtHiddenField):
                if item.type == fields.ExtHiddenField.INT:
                    val = try_to_int(val)
                elif item.type == fields.ExtHiddenField.STRING:
                    val = unicode(val)
            return val

        # список m2m полей модели нужен,
        # чтобы проверить возможность их сохранения
        try:
            list_of_m2m = [
                x[0].name for x in ModelOptions(obj).get_m2m_with_model()]
        except AttributeError:
            list_of_m2m = []

        # Присваиваем атрибутам связываемого объекта соответствующие поля формы
        all_fields = self._get_all_fields(self)
        for field in all_fields:
            if not field.name:
                continue
            field_name = str(field.name)
            assert field_name, (
                'The names of all fields must be set for a successful'
                ' assignment. Check the definition of the form.'
            )

            # заполним атрибуты только те, которые не в списке исключаемых
            if not field_name in exclusion:
                # запрещаем пытаться сохранять many2many для объекта без pk
                if hasattr(obj, 'pk'):
                    if obj.pk is None and field_name in list_of_m2m:
                        raise ValueError(' '.join(
                            ["'%s' instance needs to have a primary" % (
                                obj.__class__.__name__),
                             "key value before a many-to-many "
                             "relationship can be used."]))

                names = field_name.split('.')
                set_field(obj, names, convert_value(field), field)

    @property
    def items(self):
        return self._items

    def pre_render(self):
        from m3_ext.ui.fields import ExtHiddenField
        super(ExtForm, self).pre_render()
        if not self.focused_field:
            childs = self._get_all_fields(self)
            for child in childs:
                if _is_field(child) and not isinstance(
                        child, ExtHiddenField) and not child.hidden:
                    self.focused_field = child
                    break


#==============================================================================
class ExtPanel(BaseExtPanel):
    """
    Панель. Kак правило этот контрол включает другие компоненты для отображения
    """
    def __init__(self, *args, **kwargs):
        super(ExtPanel, self).__init__(*args, **kwargs)

        # Отступ от внешних границ
        self.padding = None

        # Возможность сворачивать панель
        self.collapsible = False

        # Показывать ли внутреннюю границу у элемента
        self.body_border = True

        # Базовый CSS класс, по умолчанию 'x-panel'
        self.base_cls = ''

        # Данное свойства - приватное в контексте extjs
        # Переопределяет стиль панели
        self.body_cls = ''

        # Автозагрузка контента
        self.auto_load = None

        self.auto_scroll = True
        # Позволять ли панели быть "плавающей"
        # (см Ext.layout.BorderLayout.Region)
        self.floatable = True
        # Сворачивать панель при щелчке на заголовке?
        self.title_collapse = False

        self.init_component(*args, **kwargs)

    def render_base_config(self):
        """
        Точная имитация рендера из покойного шаблона
        """
        super(ExtPanel, self).render_base_config()
        for args in (
            ('padding', self.padding),
            ('collapsible', self.collapsible, self.collapsible),
            ('bodyBorder', self.body_border, not self.body_border),
            ('baseCls', self.base_cls, self.base_cls),
            ('bodyCfg', {'cls': self.body_cls}, self.body_cls),
            ('autoLoad', self.auto_load),
            ('autoScroll', self.auto_scroll, self.auto_scroll),
            ('floatable', self.floatable, not self.floatable),
            ('titleCollapse', self.title_collapse, self.title_collapse),
        ):
            self._put_config_value(*args)
        if self._items:
            self._put_config_value('items', self.t_render_items)

    def render(self):
        self.pre_render()  # Тут рендерится контекст
        self.render_base_config()  # Тут конфиги
        self.render_params()  # Пусто
        base_config = self._get_config_str()
        return 'new Ext.Panel({%s})' % base_config

    def render_globals(self):
        """
        Рендерит и возвращает js-код, который помещен в template_globals
        """
        return _render_globals(self)

    @property
    def items(self):
        return self._items


#==============================================================================
class ExtTitlePanel(ExtPanel):
    """
    Расширенная панель с возможностью добавления контролов в заголовок.
    """
    def __init__(self, *args, **kwargs):
        super(ExtTitlePanel, self).__init__(*args, **kwargs)
        self.template = "ext-panels/ext-title-panel.js"
        self.__title_items = []
        self.init_component(*args, **kwargs)

    def _update_header_state(self):
        # Заголовок может быть только в том случае,
        # если есть текстовое значение,
        # либо имеются компоненты
        self.header = self.title or (
            not self.title and len(self.__title_items))

    def _on_title_after_addition(self, component):
        # Событие вызываемое после добавления элемента в заголовок
        self.items.append(component)
        self._update_header_state()

    def _on_title_before_deletion(self, component):
        # Событие вызываемое перед удалением элемента из заголовка
        self.items.remove(component)

    def _on_title_after_deletion(self, success):
        # Событие вызываемое после удаления элемента из заголовка
        self._update_header_state()

    def t_render_items(self):
        """Дефолтный рендеринг вложенных объектов."""
        return ",".join([item.render() for item in self._items if
                         item not in self.__title_items])

    def t_render_title_items(self):
        """Дефолтный рендеринг вложенных объектов заголовка."""
        return ",".join([item.render() for item in self.__title_items])

    @property
    def title_items(self):
        return self.__title_items

    def render(self):
        #WARNING!
        # Не удалось перевести этот компонент на полность питонячий рендер
        # Потому что в ЭПК шаблон этого компонента переопределяется
        # И дабы не ломать все их формы, приходится оставлять старый рендер
        # Посылаю им лучи ненависти и поноса!
        return BaseExtComponent.render(self)


#==============================================================================
class ExtTabPanel(BaseExtPanel):
    """
    Класс, отвечающий за работу TabPanel
    """

    # Возможности размещения табов
    TOP = 'top'
    BOTTOM = 'bottom'

    def __init__(self, *args, **kwargs):
        super(ExtTabPanel, self).__init__(*args, **kwargs)
        self.template = 'ext-panels/ext-tab-panel.js'

        # Активная вкладка
        self.active_tab = 0

        # Активный скрол у табов
        self.enable_tab_scroll = True

        # Внутренняя граница
        self.body_border = True

        # Если True, то применяется lazy рендеринг табов
        self.deferred_render = None

        # Если True, то layout таба перестраивается при каждой активации таба
        self.layout_on_tab_change = None

        # Показывает панель вкладок без фонового изображения
        self.plain = False

        # Если False, то TabPanel указывается фиксированная ширина,
        # либо она подчиняется layout родителя
        self.auto_width = True

        # Табы
        self._items = []

        # Позиция отображения табов: возможные варианты TOP и BOTTOM
        self.tab_position = self.TOP

        self.init_component(*args, **kwargs)

    def add_tab(self, **kwargs):
        panel = ExtPanel(**kwargs)
        self.tabs.append(panel)
        return panel

    @property
    def tabs(self):
        return self._items

    @property
    def items(self):
        return self._items


#==============================================================================
class ExtFieldSet(ExtPanel):
    '''
    Объеденяет внутренние элементы и создает рамку для остальных контролов
    '''
    def __init__(self, *args, **kwargs):
        self.checkbox_toggle = False
        # имя чекбокса, используется в случае checkboxToggle = True
        self.checkbox_name = None
        super(ExtFieldSet, self).__init__(*args, **kwargs)

    def render_base_config(self):
        super(ExtFieldSet, self).render_base_config()
        self._put_config_value('checkboxToggle', self.checkbox_toggle)
        self._put_config_value('checkboxName', self.checkbox_name)

    def render(self):
        self.pre_render()  # Тут рендерится контекст
        self.render_base_config()  # Тут конфиги
        self.render_params()  # Пусто
        base_config = self._get_config_str()
        return 'new Ext.form.FieldSet({%s})' % base_config

    @property
    def checkboxToggle(self):
        """
        deprecated
        """
        return self.checkbox_toggle

    @checkboxToggle.setter
    def checkboxToggle(self, value):
        """
        deprecated
        """
        self.checkbox_toggle = value



# -*- coding: utf-8 -*-
"""
Результаты выполнения Action`s
"""
from django import http

from m3.actions import (
    ActionResult as _ActionResult,
    BaseContextedResult as _BaseContextedResult,
)

import helpers as _helpers


class ExtUIScriptResult(_BaseContextedResult):
    """
    По аналогии с ExtUiComponentResult,
    представляет собой некоторого наследника класса ExtUiComponent.
    Единственное отличие заключается в том,
    что get_http_response должен сформировать
    готовый к отправке javascript.
    .. note::
        Т.е. должен быть вызван метод self.data.get_script()
    """
    def __init__(
            self, data=None, context=None,
            http_params=None, secret_values=False):
        super(ExtUIScriptResult, self).__init__(data, context, http_params)
        self.secret_values = secret_values

    def get_http_response(self):
        self.data.action_context = self.context
        response = http.HttpResponse(self.data.get_script())

        response = self.process_http_params(response)

        if self.secret_values:
            response['secret_values'] = True
        return response


class ExtUIComponentResult(_BaseContextedResult):
    """
    Результат выполнения операции,
    описанный в виде отдельного компонента пользовательского интерфейса.
    В self.data хранится некоторый наследник класса m3_ext_demo.ui.ExtUiComponent.
    Метод get_http_response выполняет метод render у объекта в self.data.
    """
    def get_http_response(self):
        self.data.action_context = self.context
        return http.HttpResponse(self.data.render())


class ExtGridDataQueryResult(_ActionResult):
    """
    Результат выполнения операции,
    который выдает данные в формате, пригодном для
    отображения в гриде
    """
    def __init__(self, data=None, start=-1, limit=-1):
        super(ExtGridDataQueryResult, self).__init__(data)
        self.start = start
        self.limit = limit

    def get_http_response(self):
        return http.HttpResponse(
            _helpers.paginated_json_data(
                self.data, self.start, self.limit))



#coding:utf-8
"""
Модуль с базовыми классами/интерфейсами, которые необходимы
для работы подсистемы m3_ext_demo.ui

Created on 01.03.2010

@author: akvarats
@author: prefer
"""

import datetime
import decimal

from m3_ext.ui import render_component
from m3_ext.ui import generate_client_id, normalize
from m3 import date2str

from .helpers import _render_globals


class ExtComponentException(Exception):
    """
    Ошибка М3-шного экстового компонента.
    """
    pass


class ListenerMap(object):
    """
    Дескриптор коллекции листенеров.
    При рендеринге контрола с выставленным флагом flag_attr
    обработчики из списка в атрибуте storage_attr
    будут заменены на None
    """
    def __init__(self, storage_attr, flag_attr):
        self._storage_attr = storage_attr
        self._flag_attr = flag_attr

    def __get__(self, inst, cls):
        disable = getattr(inst, self._flag_attr, False)
        manageable = set(getattr(cls, 'manageable_listeners', tuple()))
        class ListenerStorage(object):

            def __init__(self, data):
                self._data = data

            def items(self):
                if not disable:
                    return self._data.items()
                return [
                    (k, v if k not in manageable else None)
                    for (k, v) in self._data.iteritems()
                ]

            def __getitem__(self, key):
                return self._data[key]

            def __setitem__(self, key, val):
                self._data[key] = val

            def get(self, key, default=None):
                return self._data.get(key, default)

        return ListenerStorage(getattr(inst, self._storage_attr))

    def __set__(self, inst, val):
        assert inst is not None, "ListenerMap settable only in instances!"
        setattr(inst, self._storage_attr, val)


class BaseExtComponent(object):
    """
    Базовый класс для всех компонентов пользовательского интерфейса
    """

    _listeners = ListenerMap('_listeners_stored', 'read_only')

    def __init__(self, *args, **kwargs):
        # Каждый компонент может иметь шаблон, в котором он будет рендериться
        self.template = ''

        # :deprecated: Только окно (наследник BaseExtWindow)
        # может иметь данный атрибут
        self.template_globals = ''

        # Уникальный идентификатор компонента
        self.client_id = generate_client_id()

        # action context of the component (normally, this is
        # an instance of m3.actions.ActionContext class
        self.action_context = None

        # Словарь обработчиков на события
        self._listeners = {}

        # Список словарей с основной конфигурацией компонента
        self._config_list = []

        # Список словарей с дополнительной конфигурацией компонента
        self._param_list = []

        # Если True, то рендерится как функция, без префикса new
        self._is_function_render = False

        # Имя компонента в нотации ExtJs (Например Ext.form.Panel)
        self._ext_name = None

    def render(self):
        """
        Возвращает "кусок" javascript кода, который используется для
        отображения самого компонента. За рендер полного javascript
        отвечает метод get_script()
        :rtype: str
        """
        self.pre_render()
        return render_component(self)

    def render_globals(self):
        """
        Рендерит и возвращает js-код, который помещен в template_globals
        :rtype: str
        """
        return _render_globals(self)

    def pre_render(self):
        """
        Вызывается перед началом работы метода render
        """
        pass

    def init_component(self, *args, **kwargs):
        """
        Заполняет атрибуты экземпляра значениями в kwargs,
        если они проинициализированы ранее
        """
        for k, v in kwargs.items():
            assert k in dir(self) and not callable(getattr(self, k)), (
                'Instance attribute "%s" should be defined in class "%s"!'
                % (k, self.__class__.__name__))
            self.__setattr__(k, v)

    def t_render_listeners(self):
        """
        :deprecated: Если рендеринг не в шаблоне,
        то при вызове render_base_config информация о подписчиках на события
        уже будут в конфиге

        Инкапсуляция над _listeners. Используется из шаблонов!
        """
        return dict([
            (k, v) for k, v in self._listeners.items() if v is not None])

    def t_render_simple_listeners(self):
        """
        :deprecated: Конфиг должен рендериться в render_base_config
        """
        return '{%s}' % ','.join([
            '%s:%s' % (k, v)
            for k, v in self._listeners.items()
            if not isinstance(v, BaseExtComponent) and v is not None
        ])

    def render_base_config(self):
        """
        Рендерит базовый конфиг (Конфигурация extjs контрола)
        """
        self._put_config_value('id', self.client_id)
        if self._listeners:
            self._put_config_value(
                'listeners', self.t_render_simple_listeners)

    def render_params(self):
        """
        Рендерит дополнительные параметры
        (Как правило используется для разработки
        собственных контролов на базе контрола extjs)
        """
        pass

    def __check_unicode(self, string):
        """
        Проверка на не юникодную строку в которой есть русские символы
        Если есть русские символы необходимо использовать юникод!
        :param string: строка для проверки
        :type string: str / unicode
        :raise: Exception
        """
        try:
            unicode(string)
        except UnicodeDecodeError:
            raise Exception('"%s" is not unicode' % string)
        else:
            return string

    def _put_base_value(
            self, src_list, extjs_name, item, condition=True, depth=0):
        """
        Управляет правильной установкой (в зависимости от типа)
        параметров контрола
        :param src_list: список словарей
        :type src_list: list
        :param extjs_name: Оригинальное название атрибута в ExtJs
        :type: extjs_name: str
        :param item: значение атрибута в М3
        :type item: basestring, bool, int, float, decimal, long, dict
        :param condition: Условие добавления в конфиг.
        :type condition: bool
        :param depth: глубина обхода
        :type depth: int

        :raise: ExtComponentException
        """
        conf_dict = {}
        res = None
        if item is None or not condition:
            return
        elif callable(item):
            res = self.__check_unicode(item())

        elif isinstance(item, basestring):

            # если в строке уже есть апостроф, то будет очень больно.
            # поэтому replace
            res = "'%s'" % normalize(self.__check_unicode(item))

        elif isinstance(item, bool):
            res = str(item).lower()

        elif isinstance(item, (int, float, decimal.Decimal, long)):
            res = item

        elif isinstance(item, datetime.date):
            res = "'%s'" % date2str(item)

        elif isinstance(item, dict):
            # рекурсивный обход вложенных свойств
            d_tmp = {}
            for k, v in item.items():
                prop = self._put_base_value(
                    src_list=src_list,
                    extjs_name=k,
                    item=v,
                    depth=depth + 1
                )
                if prop:
                    d_tmp[k] = prop[k]
            res = d_tmp

        elif hasattr(item, '__unicode__'):
            return self._put_base_value(
                src_list, extjs_name, unicode(item), condition, depth)
        else:
            # Эээээ... Выводится для себя
            raise ExtComponentException(
                u'Тип переданного параметра не '
                u'поддерживается: "%s":"%s"' % (extjs_name, item))

        if res is not None:
            conf_dict[extjs_name] = res
            if depth == 0:
                src_list.append(conf_dict)

            return conf_dict

    def _put_config_value(self, extjs_name, item, condition=True):
        """
        Добавляет значение в конфиг компонента ExtJs,
        для последующей передачи в конструктор JS
        :param extjs_name: Оригинальное название атрибута в ExtJs
        :type: extjs_name: str
        :param item: Значение атрибута в М3
        :param condition: Условие добавления в конфиг.
        :type condition: bool

        .. note::
            Бывает полезно чтобы не использовать лишний if
        """
        self._put_base_value(self._config_list, extjs_name, item, condition)

    def _put_params_value(self, extjs_name, item, condition=True):
        """
        Обертка для упаковки детализированного конфига компонента
        :param extjs_name: Оригинальное название атрибута в ExtJs
        :type: extjs_name: str
        :param item: Значение атрибута в М3
        :param condition: Условие добавления в конфиг.
        :type condition: bool
        """
        self._put_base_value(self._param_list, extjs_name, item, condition)

    def _set_base_value(self, src_list, key, value):
        """
        Устанавливает значение по ключу
        :param src_list: список словарей структуры
        :type src_list: list
        :param key: ключ
        :param value: значение
        :raise: AssertionError
        """
        def set_value_to_dict(src_dict, key, value):
            """
            Вспомогательная функция, позволяет рекурсивно собрать все вложенные
            структуры-словари
            :param src_dict: словарь структуры
            :type src_dict: dict
            :param key: ключ
            :param value: значение
            """
            for k, v in src_dict.items():
                if isinstance(v, dict):
                    res = set_value_to_dict(v, key, value)
                    if res:
                        return True
                elif k == key:
                    if value:
                        src_dict[k] = value
                    else:
                        src_dict[k] = '""'
                    return True
            return False

        for item in src_list:
            assert isinstance(item, dict), 'Nested structure must be dict type'
            res = set_value_to_dict(item, key, value)
            if res:
                return True

        return False

    def _set_config_value(self, key, value):
        return self._set_base_value(self._config_list, key, value)

    def _set_params_value(self, key, value):
        return self._set_base_value(self._param_list, key, value)

    def _get_base_str(self, src_list):
        """
        Возвращает структуру в json-формате
        :param src_list: Список словарей, словари могут быть вложенными
        :type src_list: list

        Пример структуры:
        [
            {...},
            {...},
            {
                {...},
                {...},
            }
        ]
        """
        def get_str_from_dict(src_dict):
            """
            Вспомогательная функция, позволяет рекурсивно собрать все вложенные
            структуры-словари
            :param src_dict: структура-словарь
            :type src_dict: dict
            """
            tmp_list = []
            for k, v in src_dict.items():
                if isinstance(v, dict):
                    tmp_list.append(
                        '%s:{%s}' % (k, get_str_from_dict(v))
                    )
                else:
                    tmp_list.append(
                        '%s:%s' % (k, v)
                    )
            return ','.join(tmp_list)

        tmp_list = []
        for item in src_list:
            assert isinstance(item, dict), 'Nested structure must be dict type'
            tmp_list.append(get_str_from_dict(item))

        return ','.join(tmp_list)

    def _get_config_str(self):
        """
        Возвращает конфиг в формате json
        """
        return self._get_base_str(self._config_list)

    def _get_params_str(self):
        """
        Возрвращает доп. параметры в формате json
        """
        return self._get_base_str(self._param_list)


class ExtUIComponent(BaseExtComponent):
    """
    Базовый класс для компонентов визуального интерфейса
    Наиболее походит на BoxComponent в ExtJS
    """

    def __init__(self, *args, **kwargs):
        super(ExtUIComponent, self).__init__(*args, **kwargs)

        # Произвольные css стили для контрола
        self.style = {}

        # Будет ли отображаться
        self.hidden = False

        # Будет ли активен
        self.disabled = False

        # Высота и ширина
        self.height = self.width = None

        # Координаты по оси Х и Y для абсолютного позиционирования
        self.x = self.y = None

        # html содержимое
        self.html = None

        # Расположение компонента при использовании layout=border
        self.region = None

        # Количество занимаемых ячеек для layout=*box (vbox или hbox)
        self.flex = None

        # Максимальные и минимальные ширины и высоты
        self.max_height = self.min_height = (
            self.max_width) = self.min_width = None

        # Наименование
        self.name = None

        # Якорь
        self.anchor = None

        # CSS класс, который будет добавлен к компоненту
        self.cls = None

        # Использовать ли автоскрол
        self.auto_scroll = False

        # Включение автовысоты. Аналог height='auto'
        self.auto_height = False

        # Включение автоширины. Аналог width='auto'
        self.auto_width = False

        # Метка поля
        self.label = None

        # Скрыть label
        self.hide_label = False

        # CSS стиль для label
        self.label_style = {}

    def t_render_style(self):
        """
        :deprecated: Использовать рендеринг в render_base_config
        """
        return '{%s}' % ','.join(
            ['"%s":"%s"' % i for i in self.style.items()]
        )

    def render_base_config(self):
        """
        Рендер базового конфига объекта
        """
        super(ExtUIComponent, self).render_base_config()
        self._put_config_value('style', self.t_render_style, self.style)
        self._put_config_value('hidden', self.hidden, self.hidden)
        self._put_config_value('disabled', self.disabled, self.disabled)
        self._put_config_value('height', self.height)
        self._put_config_value('width', self.width)
        self._put_config_value('x', self.x)
        self._put_config_value('y', self.y)
        self._put_config_value('html', self.html)
        self._put_config_value('region', self.region)
        self._put_config_value('flex', self.flex)
        self._put_config_value('maxHeight', self.max_height)
        self._put_config_value('minHeight', self.min_height)
        self._put_config_value('maxWidth', self.max_width)
        self._put_config_value('minWidth', self.min_width)
        self._put_config_value('name', self.name)
        self._put_config_value('anchor', self.anchor)
        self._put_config_value('cls', self.cls)
        self._put_config_value(
            'autoScroll', self.auto_scroll, self.auto_scroll)
        self._put_config_value(
            'autoHeight', self.auto_height, self.auto_height)
        self._put_config_value('autoWidth', self.auto_width, self.auto_width)
        self._put_config_value('fieldLabel', self.label)
        if self.label_style:
            self._put_config_value('labelStyle', self.t_render_label_style())
        self._put_config_value('hideLabel', self.hide_label, self.hide_label)

    def make_read_only(
            self, access_off=True, exclude_list=(), *args, **kwargs):
        """
        :param access_off: переменная регулирует вкл\выкл режима для чтения.
        :type access_off: bool
        :param exclude_list: список, содержит в себе имена элементов,
            которые не надо выключать.
            Позволяет сделать компонент недоступным для изменения.
            Обязательно должен быть переопределен в наследуемых классах.
        :type exclude_list: list

        .. note::
        При вызове метода без параметров, используется параметр по-умолчанию
        access_off=True, в этом случае метод делает компонент, и все контролы
        расположенные на нем неизменяемыми.
        _make_read_only(False) соответственно делает компонент доступным для
        изменения, так же как и все контролы на нем.
        """
        if (
            (self in exclude_list) or
            (hasattr(self, 'name') and self.name and self.name in exclude_list)
        ):
            actual_access_off = not access_off
        else:
            actual_access_off = access_off

        return self._make_read_only(
            actual_access_off, exclude_list, *args, **kwargs)

    def _make_read_only(
            self, access_off=True, exclude_list=(), *args, **kwargs):
        raise NotImplementedError()



#coding:utf-8
"""
Created on 3.3.2010

@author: prefer
"""

from m3_ext.ui.base import ExtUIComponent


# FIXME: для чего наследуется от ExtUIComponent, а не от BaseExtComponent?
class BaseExtStore(ExtUIComponent):
    """
    Базовый класс для Store - компонента хранения данных
    """
    def __init__(self, *args, **kwargs):
        super(BaseExtStore, self).__init__(*args, **kwargs)

        # Свойства, которые будут посылаться на сервер для каждого запроса
        self._base_params = {}

        # Признак автозагрузки
        self.auto_load = False

        # Признак автосохранения при изменении данных
        self.auto_save = True

        # Ссылка для получения/сохранения данных
        self.url = ''

        # Объект, который отвечает за запись данных
        self.writer = None

    def _set_base_params(self, params):
        self._base_params.update(params)

    def _get_base_params(self):
        return self._base_params

    base_params = property(_get_base_params, _set_base_params)

    @property
    def handler_beforeload(self):
        return self._listeners.get('beforeload')

    @handler_beforeload.setter
    def handler_beforeload(self, function):
        self._listeners['beforeload'] = function
